
/*
 * $Id: ChbModel.cc,v 1.1.2.2 1999/02/21 19:38:42 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include "ChbModel.h"

ChbModel::ChbModel(): theCR(0), theDR(0) {
}

void ChbModel::ratios(double aCR, double aHR) {
	// hits are always cachable, thus we are allowed 
	// to make decisions on misses only (1-HR)
	// our negative decisions (1-DR) should result
	// in a proper uncachable ratio (1-CR):
	//     (1 - HR) * (1 - DR) = 1 - CR
    //     DR = 1 - (1 - CR)/(1 - HR) = (CR - HR)/(1 - HR)
	// a) clearly, CR must be at least HR
	// b) if HR is 1, then CR must be 1, and
	// DR can take any value since we won't have a chance 
	// to decide!

	Assert(aCR >= aHR); // see (a) above

	if (aHR >= 1)
		theDR = 1; // see (b) above
	else
		theDR = (aCR - aHR) / (1 - aHR);

	theCR = aCR;
}
