
/*
 * $Id: Histogram.cc,v 1.1.1.2 1999/02/05 16:51:34 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include "Histogram.h"
#include "ILog.h"
#include "OLog.h"
#include "gadgets.h"


Histogram::Histogram(): theValMin(-1), theValMax(-1), theBinMax(-1) {
}

Histogram::Histogram(Val aMin, Val aMax, int binCount) {
	limits(aMin, aMax, binCount);
}

void Histogram::limits(Val aMin, Val aMax, int binCount) {
	theBins.stretch(binCount+2);
	theBins.count(binCount+2);
	theValMin = aMin;
	theValMax = aMax;
	theBinMax = binCount+1;
}

void Histogram::reset() {
	theBins.memset(0);
	theValMin = theValMax = theBinMax = -1;
}

void Histogram::record(Val v) {
	theStats.record(v);
	v -= theValMin;
	const int idx = v >= 0 ? 1+Min(val2Bin(v), theBinMax) : 0;
	theBins[idx]++;
}

void Histogram::store(OLog &log) const {
	log << theValMin << theValMax << theStats << theBins;
}

void Histogram::load(ILog &log) {
	log >> theValMin >> theValMax >> theStats >> theBins;
}
