
/*
 * $Id: Progress.cc,v 1.1.2.3 1999/02/20 03:42:44 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "polygraph.h"

#include <iostream.h>

#include "ILog.h"
#include "OLog.h"

#include "Progress.h"


/* ErrRec */

ErrRec::ErrRec(): theCnt(-1), theRatio(-1) {
}

ostream &ErrRec::print(ostream &os) const {
	// either/or
	if (theRatio > 0)
		return os << theRatio;
	else
		return os << theCnt;
}


/* ProgrRec */

ProgrRec::ProgrRec(): theXactComplCnt(-1) {
}

void ProgrRec::store(OLog &ol) const {
	ol << theDuration << theXactComplCnt << theErrs.theCnt << theErrs.theRatio;
}

void ProgrRec::load(ILog &il) {
	il >> theDuration >> theXactComplCnt >> theErrs.theCnt >> theErrs.theRatio;
}

ostream &ProgrRec::print(ostream &os) const {
	return theErrs.print(os << theXactComplCnt << ':' << theDuration << ':');
}

/* Progress */

Progress::Progress() {
	theXactComplCnt = theErrs.theCnt = 0;
	theErrs.theRatio = -1;
}

void Progress::update(Time curTime) {
	Clock::update(curTime);
	if (theStartTime < 0)
		theStartTime = curTime;
	theDuration = curTime - theStartTime;
}

/* Goal */

Goal &Goal::operator /=(double factor) {
	Assert(factor > 0);

	if (theDuration > 0)
		theDuration /= factor;
	if (theXactComplCnt > 0)
		theXactComplCnt = (int) (theXactComplCnt/factor);

	if (theErrs.theCnt >= 0)
		theErrs.theCnt = (int) (theErrs.theCnt/factor);
	else
	if (theErrs.theRatio > 0)
		theErrs.theRatio /= factor;

	return *this;
}

Goal::operator void*() const {
	// note: errors alone do not count as a real goal
	return theDuration >= 0 || theXactComplCnt >= 0 ?
		(void*)-1 : 0;
}
