
/*
 * $Id: LevelStat.h,v 1.1.1.2 1999/02/12 23:15:34 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#ifndef LEVEL_STAT_H
#define LEVEL_STAT_H

#include "Time.h"

class OLog;
class ILog;

// [concurrency] level statistics
class LevelStat {
	public:
		LevelStat();

		LevelStat &operator ++() { inc(); return *this; }
		LevelStat &operator --() { dec(); return *this; }

		// XXX: updates level but does not change the decCnt
		// perhaps we should switch to attempts/successes/level
		void failure() { change(); theLevel--; }

		int incCnt() const { return theIncCnt; }
		int decCnt() const { return theDecCnt; }
		int level() const { return theLevel; } 
		double mean() const;

		void restart();
		void store(OLog &log) const;
		void load(ILog &log);

	protected:
		void inc() { change(); theIncCnt++; theLevel++; }
		void dec() { change(); theDecCnt++; theLevel--; }
		void change();

	protected:
		int theIncCnt;
		int theDecCnt;
		int theLevel;     // internal, not reset on restart

		Time theStart;    // start of measurements
		Time theCurStart; // start of the current level
		Time theSum;      // "level-seconds" (nominator of an average)
};


inline OLog &operator <<(OLog &log, const LevelStat &ls) { ls.store(log); return log; }
inline ILog &operator >>(ILog &log, LevelStat &ls) { ls.load(log); return log; }


#endif
