
/*
 * $Id: String.cc,v 1.1.1.5 1999/02/07 02:18:00 rousskov Exp $
 *
 *
 * Web Polygraph  http://www.ircache.net/Polygraph/
 * ----------------------------------------------------------
 *
 *  Web Polygraph is copyrighted(C) 1998 by Alex Rousskov and
 *  the University of California, San Diego (UCSD), with all 
 *  rights reserved. Development is funded by the National Science
 *  Foundation (NLANR Cache grants NCR 9616602 and NCR 9521745).
 *  Please see the COPYRIGHT file for full details.
 *
 *  Web Polygraph is free software. You may copy and distribute 
 *  VERBATIM copies of this product. You may modify your copy 
 *  for your own use. You must NOT distribute modified copies.
 *
 *  If you publish any materials derived from or related to  
 *  Web Polygraph, you must give a credit reference to the
 *  National Science Foundation and the National Laboratory 
 *  for Applied Network Research.
 *
 *  This program is distributed WITHOUT ANY WARRANTY;
 *  without even the implied warranty of MERCHANTABILITY or
 *  FITNESS FOR A PARTICULAR PURPOSE.
 *
 */

#include "xstd.h"

#include <iostream.h>

#include "Assert.h"
#include "String.h"
#include "gadgets.h"

/* StrBuf */

void StrBuf::set(const char *aBuf, int aSize) {
	Assert(!theUseLvl);
	//cerr << here << "new string: " << aSize << ' ' << aBuf << endl;
	delete[] theBuf;
	alloc(aSize);
	memcpy(theBuf, aBuf, aSize);
}

StrBuf *StrBuf::clone() const {
	StrBuf *sb = new StrBuf;
	sb->set(theBuf, theSize);
	return sb;
}

/* String */

void String::set(const char *s) {
	if (theBuf)
		theBuf = theBuf->abandon();
	if (s) {
		theBuf = new StrBuf;
		theBuf->set(s, strlen(s)+1);
		theBuf->use();
	}
}

void String::set(const String &s) {
	if (theBuf)
		theBuf->abandon();
	if ((theBuf = s.theBuf))
		theBuf->use();
}

void String::alloc(int size) {
	if (theBuf)
		theBuf->abandon();
	theBuf = new StrBuf;
	theBuf->alloc(size);
}

// clone a private copy if we are using shared memory
// (copy-on-write or lazy copying)
void String::aboutToMod() {
	if (theBuf && theBuf->theUseLvl > 1) {
		StrBuf *sb = theBuf;
		theBuf = sb->clone();
		sb->abandon();
	}
}

String String::operator ()(int min, int maxp1) const {
	StrBuf *sb = new StrBuf;
	const int len = maxp1 - min;
	Assert(len >= 0);
	sb->alloc(len+1);
	if (len)
		memcpy(sb->theBuf, str()+min, len);
	sb->theBuf[len] = '\0';
	return *sb;
}

bool String::operator ==(const String &s) const {
	return 
		theBuf && s.theBuf &&
		theBuf->theBuf && s.theBuf->theBuf &&
		!strcmp(theBuf->theBuf, s.theBuf->theBuf);
}

ostream &operator <<(ostream &os, const String &s) {
	if (s) {
		if (s.len() > 0)
			os.write(s.str(), s.len());
	} else
		os << "<none>";
	return os;
}
